<?php

namespace App\domain\amadeus;

use App\domain\amadeus\traits\AmadeusParamsTraits;
use App\Traits\FileStorageTrait;
use Carbon\Carbon;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;

class FlightSearchAction
{
    use AmadeusParamsTraits;
    use FileStorageTrait;


    protected string $searchCriteriaString;
    protected int $numberOfPassengers;
    protected string $passengerString;

    public function execute(array $passengerCriteria, array $searchCriteriaFlight, ?array $searchModifiers)
    {
        try {
            ;
            $this->searchCriteriaString = $this->generateSearchCriteria($searchCriteriaFlight);
            $this->passengerString = $this->generatePassengerCriteria($passengerCriteria);

            $amadeusHeaders = new AmadeusHeaders();
            ['nonce' => $nonce, 'utcTimestamp' => $timestamp, 'mid' => $mid, 'passSHA' => $passSHA] = $amadeusHeaders->getHeaders();

            $action = "http://webservices.amadeus.com/FMPTBQ_23_2_1A";

            $curl = curl_init();

            $caBundlePath = base_path('cacert.pem');

            curl_setopt_array($curl, array(
                CURLOPT_URL => $this->getToAddress(),
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'POST',
                CURLOPT_POSTFIELDS => $this->generateXMLRequest($mid, $action, $nonce, $passSHA, $timestamp),
                CURLOPT_HTTPHEADER => array(
                    'Content-Type: text/xml; charset=utf-8',
                    'SOAPAction: ' . $action . ''
                ),
                CURLOPT_CAINFO => $caBundlePath,
            ));

            $response = curl_exec($curl);

            self::storeInFile("xml", "0-flight-search-response", $response);


//            Log::info("Search Response: ". $response);

            if (curl_errno($curl)) {
                dd('Curl error: ' . curl_error($curl));
            }

            curl_close($curl);

            $xmlObject = simplexml_load_string($response);

//            // Register the 'awsse' namespace
//            $xmlObject->registerXPathNamespace('awsse', 'http://xml.amadeus.com/2010/06/Session_v3');
//
//            $sessionId = (string)$xmlObject->xpath('//awsse:Session/awsse:SessionId')[0];
//            $sequenceNumber = (string)$xmlObject->xpath('//awsse:Session/awsse:SequenceNumber')[0];
//            $securityToken = (string)$xmlObject->xpath('//awsse:Session/awsse:SecurityToken')[0];
//
//            $expirationTime = 24 * 60;
//
//            Session::put('amadeus-session-id', $sessionId, $expirationTime);
//            Session::put('amadeus-sequence-number', $sequenceNumber, $expirationTime);
//            Session::put('amadeus-security-token', $securityToken, $expirationTime);

            $body = $xmlObject->xpath('soap:Body');

            self::storeInFile("json","0-flight-search-response",json_encode($body));

            return $this->xmlToArray($body);

        } catch (\Exception $exception) {
            $e = "Error in Amadeus Search Offer Request:0" . $exception->getMessage();
            Log::error($e);
            echo $e;
        }
    }

    protected function generateXMLRequest($mid, string $action, $nonce, $passSHA, $timestamp): string
    {
        $xml = '<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:ses="http://xml.amadeus.com/2010/06/Session_v3">
  <soap:Header>

    <add:MessageID xmlns:add="http://www.w3.org/2005/08/addressing">' . $mid . '</add:MessageID>
    <add:Action xmlns:add="http://www.w3.org/2005/08/addressing">' . $action . '</add:Action>
    <add:To xmlns:add="http://www.w3.org/2005/08/addressing">' . $this->getToAddress() . '</add:To>
    <link:TransactionFlowLink xmlns:link="http://wsdl.amadeus.com/2010/06/ws/Link_v1" />
    <oas:Security xmlns:oas="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd">
      <oas:UsernameToken xmlns:oas1="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd" oas1:Id="UsernameToken-1">
        <oas:Username>WSCRLCAR</oas:Username>
        <oas:Nonce EncodingType="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-soap-message-security-1.0#Base64Binary"> ' . $nonce . '</oas:Nonce>
        <oas:Password Type="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordDigest">' . $passSHA . '</oas:Password>
        <oas1:Created>' . $timestamp . '</oas1:Created>
      </oas:UsernameToken>
    </oas:Security>
    <AMA_SecurityHostedUser xmlns="http://xml.amadeus.com/2010/06/Security_v1">
      <UserID AgentDutyCode="SU" POS_Type="1" PseudoCityCode="LONU128OS" RequestorType="U" />
    </AMA_SecurityHostedUser>
  </soap:Header>

<soap:Body>
    <Fare_MasterPricerTravelBoardSearch>
      <numberOfUnit>
        <unitNumberDetail>
          <numberOfUnits>30</numberOfUnits>
          <typeOfUnit>RC</typeOfUnit>
        </unitNumberDetail>
        <unitNumberDetail>
          <numberOfUnits>' .

            $this->numberOfPassengers

            . '</numberOfUnits>
          <typeOfUnit>PX</typeOfUnit>
        </unitNumberDetail>
      </numberOfUnit>' .

            $this->passengerString

            . '<fareOptions>
        <pricingTickInfo>
          <pricingTicketing>
            <priceType>ET</priceType>
            <priceType>RP</priceType>
            <priceType>RU</priceType>
          </pricingTicketing>
        </pricingTickInfo>
      </fareOptions>
      <travelFlightInfo />' .

            $this->searchCriteriaString

            . '</Fare_MasterPricerTravelBoardSearch>
  </soap:Body>

</soap:Envelope>';

//        Log::info("SearchRequest: " . $xml);

        self::storeInFile("xml", "0-flight-search-request", $xml);

        return $xml;
    }

    protected function xmlToArray($xml)
    {
        $array = json_decode(json_encode($xml), true);

        //        $json = json_encode($arrayResult);
//        return json_decode($json, true);
        return $array;
    }

    protected function generatePassengerCriteria(array $passengerCriteria): string
    {
        $adt = 0;
        $chd = 0;
        $inf = 0;

        foreach ($passengerCriteria as $criterion) {
            if ($criterion['passengerTypeCode'] === "ADT") {
                $adt++;
            } else if ($criterion['passengerTypeCode'] === "CHD") {
                $chd++;

            } else {
                $inf++;
            }
        }

        $this->numberOfPassengers = $adt + $chd;

        $criteriaString = $this->createPTCWiseString($adt, "ADT", 1);

        if ($chd > 0) {
            $criteriaString .= $this->createPTCWiseString($chd, "CHD", $adt + 1);
        }

        if ($inf > 0) {
            $criteriaString .= $this->createInfantString($inf);
        }

        return $criteriaString;
    }

    protected function createPTCWiseString(int $passengerCount, string $ptc, int $startNumber): string
    {
        $s = "<paxReference><ptc>$ptc</ptc>";

        for ($x = $startNumber; $x < $passengerCount + $startNumber; $x++) {
            $s .= "<traveller><ref>$x</ref></traveller>";
        }

        $s .= "</paxReference>";

        return $s;
    }

    protected function createInfantString(int $infantCount): string
    {
        $s = "<paxReference><ptc>INF</ptc>";

        for ($x = 1; $x <= $infantCount; $x++) {
            $s .= "<traveller><ref>$x</ref><infantIndicator>1</infantIndicator></traveller>";
        }

        $s .= "</paxReference>";

        return $s;
    }

    protected function generateSearchCriteria(array $searchCriteriaFlight): string
    {
        $xmlSearchNode = "";

        foreach ($searchCriteriaFlight as $index => $criteria) {
            list($from, $to, $departure) = $this->getSearchCriteria($criteria);

            $refNo = $index + 1;

            $xmlSearchNode .= "
                    <itinerary>
                        <requestedSegmentRef>
                          <segRef>$refNo</segRef>
                        </requestedSegmentRef>
                        <departureLocalization>
                          <departurePoint>
                            <locationId>$from</locationId>
                          </departurePoint>
                        </departureLocalization>
                        <arrivalLocalization>
                          <arrivalPointDetails>
                            <locationId>$to</locationId>
                          </arrivalPointDetails>
                        </arrivalLocalization>
                        <timeDetails>
                          <firstDateTimeDetail>
                            <date>$departure</date>
                          </firstDateTimeDetail>
                        </timeDetails>
                    </itinerary>
            ";
        }


        return $xmlSearchNode;
    }

    protected function getSearchCriteria(array $searchCriteriaFlight): array
    {
        $from = $searchCriteriaFlight['From']['value'];
        $to = $searchCriteriaFlight['To']['value'];
        $departureCarbonDate = Carbon::parse($searchCriteriaFlight['departureDate']);
        $departure = $departureCarbonDate->format("d") . $departureCarbonDate->format("m") . $departureCarbonDate->format("y");
        return array($from, $to, $departure);
    }


}
