<?php

namespace App\domain\amadeus;

use App\Traits\FileStorageTrait;
use Illuminate\Support\Facades\Log;

class QueryFareCheckRulesAction extends AmadeusRequestInSeries
{
    use FileStorageTrait;
    protected int $sequenceNumber;
    protected string $amadeusSessionId;
    protected string $amadeusSecurityToken;

    public function __construct()
    {
        parent::__construct();

        $this->sequenceNumber = 3;
    }

    public function execute(string $amadeusSessionId, string $amadeusSecurityToken): array
    {
        try {
            $this->amadeusSessionId = $amadeusSessionId;
            $this->amadeusSecurityToken = $amadeusSecurityToken;

            $response = $this->createCurlRequest();

            self::storeInFile("xml", "1.2-fare-check-rules-response", $response);

            $responseArray = $this->xmlResponseBodyToArray($response);

            self::storeInFile(
                "json", "1.2-fare-check-rules-response", json_encode($responseArray)
            );


            return $responseArray;

        } catch (\Exception $exception) {
            $e = "Error in QueryFareCheckRulesAction execute:" . $exception->getMessage();
            Log::error($e);
            echo $e;

            return ['error' => $e];
        }
    }

    protected function generateXMLRequest(): string
    {
        $xmlRequest =  $this->xmlHeader($this->sequenceNumber, $this->amadeusSessionId, $this->amadeusSecurityToken) .
            "
            <soap:Body>

               <Fare_CheckRules>
                  <msgType>
                     <messageFunctionDetails>
                        <messageFunction>712</messageFunction>
                     </messageFunctionDetails>
                  </msgType>
                  <itemNumber>
                     <itemNumberDetails>
                        <number>1</number>
                     </itemNumberDetails>
                     <itemNumberDetails>
                        <number>2</number>
                        <type>FC</type>
                     </itemNumberDetails>
                  </itemNumber>
                  <fareRule>
                     <tarifFareRule>
                        <ruleSectionId>10</ruleSectionId>
                     </tarifFareRule>
                  </fareRule>
               </Fare_CheckRules>

            </soap:Body>

        </soap:Envelope>
        ";

        self::storeInFile("xml", "1.2-fare-check-rules-request", $xmlRequest);

        return $xmlRequest;
    }

    protected function action(): string
    {
        return "http://webservices.amadeus.com/FARQNQ_07_1_1A";
    }
}
