<?php

namespace App\domain\amadeus;

use App\Traits\FileStorageTrait;
use Illuminate\Support\Facades\Log;

class queryFarePricePNRWithBookingClassAction  extends AmadeusRequestInSeries
{
    use FileStorageTrait;
    protected string $otherCompany;
    protected int $sequenceNumber;
    protected string $amadeusSessionId;
    protected string $amadeusSecurityToken;

    public function __construct()
    {
        parent::__construct();

        $this->sequenceNumber = 4;
    }

    public function execute(string $otherCompany, string $amadeusSessionId, string $amadeusSecurityToken): array
    {
        try {
            $this->otherCompany = $otherCompany;
            $this->amadeusSessionId = $amadeusSessionId;
            $this->amadeusSecurityToken = $amadeusSecurityToken;

            $response = $this->createCurlRequest();

            self::storeInFile("xml", "2.4-Fare_PricePNRWithBookingClass_response", $response);

            $responseArray = $this->xmlResponseBodyToArray($response);

            self::storeInFile(
                "json", "2.4-Fare_PricePNRWithBookingClass_response", json_encode($responseArray)
            );

            return $responseArray;

        } catch (\Exception $exception) {
            $e = "Error in Fare_PricePNRWithBookingClass execute:" . $exception->getMessage();
            Log::error($e);
            echo $e;

            return ['error'=>$e];
        }
    }

    protected function generateXMLRequest(): string
    {
        $xmlRequest = $this->xmlHeader($this->sequenceNumber, $this->amadeusSessionId, $this->amadeusSecurityToken) .
            "
                <soap:Body>

                <Fare_PricePNRWithBookingClass>
                  <pricingOptionGroup>
                    <pricingOptionKey>
                      <pricingOptionKey>RP</pricingOptionKey>
                    </pricingOptionKey>
                  </pricingOptionGroup>
                  <pricingOptionGroup>
                    <pricingOptionKey>
                      <pricingOptionKey>RU</pricingOptionKey>
                    </pricingOptionKey>
                  </pricingOptionGroup>
                  <pricingOptionGroup>
                    <pricingOptionKey>
                      <pricingOptionKey>RLO</pricingOptionKey>
                    </pricingOptionKey>
                  </pricingOptionGroup>
                  <pricingOptionGroup>
                    <pricingOptionKey>
                      <pricingOptionKey>VC</pricingOptionKey>
                    </pricingOptionKey>
                    <carrierInformation>
                      <companyIdentification>
                        <otherCompany>" . $this->otherCompany . " </otherCompany>
                      </companyIdentification>
                    </carrierInformation>
                  </pricingOptionGroup>
                  <pricingOptionGroup>
                    <pricingOptionKey>
                      <pricingOptionKey>FCO</pricingOptionKey>
                    </pricingOptionKey>
                    <currency>
                      <firstCurrencyDetails>
                        <currencyQualifier>FCO</currencyQualifier>
                        <currencyIsoCode>GBP</currencyIsoCode>
                      </firstCurrencyDetails>
                    </currency>
                  </pricingOptionGroup>
                </Fare_PricePNRWithBookingClass>

                </soap:Body>

            </soap:Envelope>
        ";

        self::storeInFile("xml", "2.4-Fare_PricePNRWithBookingClass-request", $xmlRequest);

        return $xmlRequest;    }

    protected function action(): string
    {
        return "http://webservices.amadeus.com/TPCBRQ_21_1_1A";
    }
}
